//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//--------------------------------------------------------------------------


// ectimer.c
#include "main.h"


/*
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Function: Int_Timer0
;
; Desc:     Process a Timer 0 interrupt. Timer 0 is running at all times and
;           is used for general system timing.  It is pre-scaled to a 1kHz
;           tick rate.
;
; Setup:    Timer in Mode 1 (16 bit) (TMOD = XXXX0001B)
;           Pre-scaler to Divide-by-12 (CKCON.3 = 0)
;           Timer 0 Interrupt Enabled
;           Timer Run (TR0) = High
;
*/
void timer0(void) interrupt 1
{
   TH0 = ((TIMER0_1MSEC_RELOAD)>>8)&0x0FF;
   TL0 = (TIMER0_1MSEC_RELOAD)&0x0FF;
   if(!(--milliseconds))
   {
      milliseconds = 250;
      if(!(--QtrSeconds))
      {
         QtrSeconds = 4;
         SecBit = !SecBit;
         if(secTimerRun)
         {
            if(!(--TmrSeconds))
               TimeOut = 1;
         }
      }
   }

   if(msTimerRun)
   {
      msCounter++;
      if(msCounter==0)
      {
         msTimerOverflow = 1;
      }
   }
}


// generates a microsecond delay of the specified amount of time
void usDelay(unsigned int delay)
{
#if OSCILLATOR==33000000
   dbyte tempCounter;
   do
   {
      tempCounter = 4; // = 3 cycles
      while(--tempCounter); // = 5 cycles * 4 = 20 cycles
   }
   while(--delay); // = 10/11 cycles
   // total = (3+20+[10/11]) = [33/34] cycles
   // total = [.99/1.20] microseconds
#elif OSCILLATOR==22118400
   dbyte tempCounter;
   do
   {
      tempCounter = 2; // = 3 cycles
      while(--tempCounter); // = 5 cycles * 2 = 10 cycles
   }
   while(--delay); // = 10/11 cycles
   // total = (3+10+[10/11]) = [23/24] cycles
   // total = [1.04/1.08] microseconds
#elif OSCILLATOR==11059200
   do
   {
      ACC = 0; // 1 cycle
   }
   while(--delay); // = 10 cycles
   // total = 11 cycles = 1.10 microseconds
#endif
}

void msDelay(unsigned int delay)
{
   do
   {
      usDelay(1000);
   }
   while(--delay);
}

/*
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Function:	Wait2ms
;
; Desc:		Delay for 2 ms
;
; Note:		Xtal is 14.7456 MHz, cycle time depends on CPU type.
;			DS5000 (/12) = 814 ns, DS87C520 (/4) = 271 ns.
;
; DS89C420 (/1) @ 11.095MHz = 90 ns, 2ms = 22,222 instruction cycles
; DS89C420 (/1) @ 22.190MHz = 45 ns, 2ms = 44,444 instruction cycles
; DS89C420 (/1) @ 33.000MHz = 30 ns, 2ms = 66,666 instruction cycles
*/
void wait2ms(void)
{
   msDelay(2);
}

void wait10ms(void)
{
   msDelay(10);
}

void wait250ms(void)
{
   msDelay(250);
}




