//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//--------------------------------------------------------------------------

 // file ecash.c

#include "main.h"
#include "I2CMaster.h"

uchar OWReset();
uchar OWReadByte();
void OWWriteByte(uchar wrByte);
uchar OWSearch(uchar resetSearch, uchar *lastDevice, uchar *deviceAddress);
uchar OWWriteBit(uchar value);
uchar OWReadBit();
uchar OWBlock (uchar *block, uchar len);
uchar OWSpeed(uchar speed, uchar state_config);
uchar OWLevel(uchar level, uchar state_config);
uchar OWReadBitPower(uchar config_byte);
uchar OWWriteBytePower(uchar config_byte, uchar wrbyte);

void lcd(uchar byte);
uchar owTriplet(uchar* dir, uchar* firstBit, uchar* secondBit);
uchar LastDiscrepancy;


bit InitIIC(uchar i2cClockDelay, uchar slaveAddr)
{
   uchar test[1] = {0x00};
   I2CBus_init();
   I2CBus_setClockDelay(i2cClockDelay);
   I2CBus_setSlaveAddress(slaveAddr);
   // see if part responds to slave address
   return I2CBus_write(test,1);
}

void main(void)
{
   uchar snum[8];
   uchar lastD;
   int i;

#if PROCESSOR==DS89C420
   // enable internal 1k SRAM
   PMR |= 0x01;
#ifdef CTM2
   // start the crystal multiplier
   PMR |= 0x10;
   // wait until xtal multiplier starts
   while(!(EXIF&0x08))
      ; // no-op
   // Activate the 2X clock mode
   PMR &= 0x1F;
#endif
#endif

   // initialize the LCD
   lcd_init();

   lcd_clear();
		
   // initialize the I2C port
   if(InitIIC(2,0x30))
   {
      lcd_putchar('f');
   }
    
   OWReset();  
   
   do
   {
      OWSearch(FAILURE,&lastD,snum);
	  lcd(snum[0]);
	  lcd(snum[1]);
	  lcd(snum[2]);
	  lcd(snum[3]);
	  lcd(snum[4]);
	  lcd(snum[5]);
	  lcd(snum[6]);
	  lcd(snum[7]);
	  
      for(i=0;i<128;i++)
	     msDelay(10);
   }
   while(lastD != 1);

}

// owWriteBit
//
// Writes a single bit to 1-Wire using the DS2482.
//
// value - bit to be written to the 1-Wire (0 or 1)
uchar OWWriteBit(uchar value)
{
   uchar buff[3];
   uchar test;

   buff[0] = onewireBitCommand;   // 1-Wire bit command
   
   if(value)
   {
      buff[1] = 0xFF;
   }
   else
   {
      buff[1] = 0x7F;
   }

   if(!I2CBus_write(&buff[0],2))
   {
      lcd_putchar('f');
   }

   // checking if 1-Wire busy
   for(;;)
   {
      if(!I2CBus_read(&buff[0],1))
      {
         lcd_putchar('f');
      }

      test = buff[0] | 0xFE;

	  if(test == 0xFE)
	  {
	     break;
	  }
   }

   return SUCCESS;
}

// owReadBit
//
// Returns 0 or 1 for the bit read.
uchar OWReadBit()
{
   uchar buff[3];

   OWWriteBit(1);

   buff[0] = setReadPointerCommand;
   buff[1] = statusRegister;

   if(!I2CBus_write(&buff[0],2))
   {
      lcd_putchar('f');
   }
   else if(!I2CBus_read(&buff[2],1))
   {
	  lcd_putchar('f');
   }

   if(buff[2] & 0x20)
   {
      return SUCCESS;	
   }
   else
   {
      return FAILURE;
   }
}

// readOneWireByte
//
// Reads a 1-Wire byte using I2C commands to the DS2482.
//
// returns the byte read
//
uchar OWReadByte()
{
   uchar buff[4];
   uchar chk;

   buff[1] = setReadPointerCommand;
   buff[2] = statusRegister;

   if(!I2CBus_write(&buff[1],2))
   {
      lcd_putchar('f');
   }

   // checking if 1-Wire busy
   for(;;)
   {
      if(!I2CBus_read(&buff[0],1))
      {
         lcd_putchar('f');
      }

      chk = buff[0] | 0xFE;

	  if(chk == 0xFE)
	  {
	     break;
	  }
   }

   buff[0] = readByteCommand;
   buff[1] = setReadPointerCommand;
   buff[2] = readDataRegister;

   // read byte
   if(!I2CBus_write(&buff[0],1))
   {
      lcd_putchar('f');
   }

   // checking if 1-Wire busy
   for(;;)
   {
      if(!I2CBus_read(&buff[0],1))
      {
         lcd_putchar('f');
      }

      chk = buff[0] | 0xFE;

	  if(chk == 0xFE)
	  {
	     break;
	  }
   } 

   if(!I2CBus_write(&buff[1],2))
   {
      lcd_putchar('f');
   }
   else if(!I2CBus_read(&buff[3],1))
   {
	  lcd_putchar('f');
   }

   return buff[3];
}

// writeOneWireByte
//
// Writes a 1-Wire byte using I2C commands sent to the DS2482.
//
// wrByte - The byte to be written to the 1-Wire.
//
void OWWriteByte(uchar wrByte)
{
   uchar buff[4];
   uchar chk;

   buff[1] = setReadPointerCommand;
   buff[2] = statusRegister;

   if(!I2CBus_write(&buff[1],2))
   {
      lcd_putchar('f');
   }

   // checking if 1-Wire busy
   for(;;)
   {
      if(!I2CBus_read(&buff[0],1))
      {
         lcd_putchar('f');
      }

      chk = buff[0] | 0xFE;

	  if(chk == 0xFE)
	  {
	     break;
	  }
   }

   buff[0] = writeByteCommand;
   buff[1] = wrByte;  // read rom command

   if(!I2CBus_write(&buff[0],2))
   {
      lcd_putchar('f');
   }

   // checking if 1-Wire busy
   for(;;)
   {
      if(!I2CBus_read(&buff[0],1))
      {
         lcd_putchar('f');
      }

      chk = buff[0] | 0xFE;

	  if(chk == 0xFE)
	  {
	     break;
	  }
   } 
}

// resetOneWire
//
// Resets the 1-Wire using I2C through the DS2482.
//
// returns SUCCESS or FAILURE
//
uchar OWReset()
{
   uchar buff[5];
   uchar test;

   buff[0] = resetOneWireCommand;

   if(!I2CBus_write(&buff[0],1))
   {
      return FAILURE;
   }

   // checking if 1-Wire busy
   for(;;)
   {
      if(!I2CBus_read(&buff[0],1))
      {
         return FAILURE;
      }

      test = buff[0] | 0xFE;

	  if(test == 0xFE)
	  {
	     break;
	  }
   }

   // checking for presence pulse detect
   for(;;)
   {
      if(!I2CBus_read(&buff[0],1))
      {
         return FAILURE;
      }

      test = buff[0] | 0xFC;

      if(test == 0xFE)
	  {
	     break;
	  }
   }

   return SUCCESS;

}

// OWSearch
//
// Does a 1-Wire search for a 1-Wire device while keeping track so
// the next time this function is called the next 1-Wire device 
// serial number will be returned.  
//
// resetSearch - Resets the search parameters
// lastDevice  - Returns 1 if the search returns the last serial number
// deviceAddress - The serial number of the 1-Wire device found.
//
// returns SUCCES or FAILURE
//
uchar OWSearch(uchar resetSearch, uchar *lastDevice, uchar *deviceAddress)
{
   uchar retVal = FAILURE;
   uchar bit_number = 1;
   uchar last_zero = 0;
   uchar serial_byte_number = 0;
   uchar serial_byte_mask = 1;
   uchar firstBit, secondBit, dir;
   uchar i = 0;

   if(resetSearch)
   {
      lastDevice = 0;
	  LastDiscrepancy = 0;
   }

   // if the last call was not the last one
   if (!(*lastDevice))
   {
      // reset the 1-wire
      // if there are no parts on 1-wire, return FALSE
      if(!OWReset())
      {
         // reset the search
         lastDevice = 0;
         LastDiscrepancy = 0;
         return FAILURE;
      }

      // Issue the Search ROM command
      OWWriteByte(0xF0);

      // loop to do the search
      do
      {
         if (bit_number < LastDiscrepancy)
         {
            if(SearchSerialNum[serial_byte_number] & serial_byte_mask)
               dir = 1;
            else
               dir = 0;
         }
         else
         {
            // if equal to last pick 1, if not then pick 0
            if(bit_number==LastDiscrepancy)
               dir = 1;
            else
               dir = 0;
         }

         if(!owTriplet(&dir, &firstBit, &secondBit))
         {
            return FAILURE;
         }

         // if 0 was picked then record its position in LastZero
         if (firstBit==0 && secondBit==0 && dir == 0)
		 {
            last_zero = bit_number;
		 }

         // check for no devices on 1-wire
         if (firstBit==1 && secondBit==1)
            break;

         // set or clear the bit in the SerialNum byte serial_byte_number
         // with mask serial_byte_mask
         if (dir == 1)
		 {
            SearchSerialNum[serial_byte_number] |= serial_byte_mask;
		 }
         else
            SearchSerialNum[serial_byte_number] &= ~serial_byte_mask;

         // increment the byte counter bit_number
         // and shift the mask serial_byte_mask
         bit_number++;
         serial_byte_mask <<= 1;

         // if the mask is 0 then go to new SerialNum[portnum] byte serial_byte_number
         // and reset mask
         if (serial_byte_mask == 0)
         {
            serial_byte_number++;
            serial_byte_mask = 1;
         }
      }
      while(serial_byte_number < 8);  // loop until through all SerialNum[portnum] bytes 0-7

      retVal = FAILURE;
      // if the search was successful then
      if (bit_number == 65)//|| crcl))
      {
         // search successful so set LastDiscrepancy[portnum],LastDevice[portnum],next_result
         LastDiscrepancy = last_zero;
         if(LastDiscrepancy==0)
            *lastDevice = SUCCESS;
		 else
		    *lastDevice = FAILURE;

         for(i=0; i<8; i++)
		 {
            deviceAddress[i] = SearchSerialNum[i];
		 }

         return SUCCESS;
      }
   }

   // if no device found then reset counters so next 'next' will be
   // like a first
   if (!retVal || !SearchSerialNum[0])
   {
      LastDiscrepancy = 0;
      *lastDevice = FAILURE;
      retVal = FAILURE;
   }

   return retVal;
}

// owTriplet
//
// Performs the 1-Wire triplet command of the DS2482.
//
// dir - The direction of the bit for the serial number
// first - The first bit returned from the triplet command
// second - The second bit returned from the triplet command
//
// returns SUCCES or FAILURE
//
uchar owTriplet(uchar *dir, uchar *firstBit, uchar *secondBit)
{
   uchar buff[3];
   uchar test;

   buff[0] = 0x78;

   if(*dir>0)
      *dir = (uchar)0xFF;

   buff[1] = *dir;

   if(!I2CBus_write(&buff[0],2))
   {
      lcd_putchar('f');
   }

   if(!I2CBus_read(&buff[2],1))
   {
      return FAILURE;
   }
   else
   {
      test = buff[2] & 0x20;

	  if(test == 0x20)
	     *firstBit = 1;
	  else
	     *firstBit = 0;

	  test = buff[2] & 0x40;

	  if(test == 0x40)
	     *secondBit = 1;
	  else
	     *secondBit = 0;

	  test = buff[2] & 0x80;

	  if(test == 0x80)
	  {
	     *dir = 1;
	  }
	  else
	     *dir = 0;

      return SUCCESS;
   }

   return FAILURE;
}

// OWBlock - writes/reads a block of data
// block - block of data
// return - Success or failure of the operation.
uchar OWBlock (uchar *block, uchar len)
{
   uchar buffer[2];
   int i;

   for(i=0;i<len;i++)
   {
      OWWriteByte(block[i]);

      buffer[0] = setReadPointerCommand;    // The read pointer value is 0xE1
      buffer[1] = readDataRegister;         // The read data register value is 0xE1

      if(!I2CBus_write(&buffer[0],2))
      {
         return FAILURE;
      }
      else
         I2CBus_read(&block[i],1);
   }

   return SUCCESS;
}

// OWSpeed - changes the 1-Wire speed to normal or overdrive.
//           A Overdrive match rom or overdrive skip rom will be needed.
//
// speed - overdrive (Overdrive) or standard (Standard) speed.
// state_config - The current configuration byte settings.
//
// return - success or failure of the operation.
//
uchar OWSpeed(uchar speed, uchar state_config)
{
   uchar buffer[2];

   buffer[0] = writeConfigCommand;
   
   if(speed == Overdrive)
      buffer[1] = (state_config | 0x08) & 0x7F;
   else
      buffer[1] = (state_config | 0x80) & 0xF7;

   if(!I2CBus_write(&buffer[0],2))
   {
      return FAILURE;
   }

   return SUCCESS;
}      

// OWLevel
//
// level - Normal or Power Delivery mode
// state_config - The current configuration settings
//
// Returns if the operation was Successful or not.
//
uchar OWLevel(uchar level, uchar state_config)
{
   uchar buffer[2]; 

   buffer[0] = writeConfigCommand;

   // NORMAL = 0, POWER_DELIV = 1
   if(level == NORMAL) 
      buffer[1] = (state_config | 0x01) & 0xEF;
   else
      buffer[1] = (state_config | 0x10) & 0xFE;     

   if(!I2CBus_write(buffer,2))
   {
      return FAILURE;
   }

   return SUCCESS;
}

// OWReadBitPower 
//
// config_byte - current configuration settings
// delay - ms delay used before disabling active pullup
//
// Returns the bit information read.
//
uchar OWReadBitPower(uchar config_byte)
{ 
   uchar buffer[2];
   
   buffer[0] = writeConfigCommand;
   buffer[1] = (config_byte | 0x04) & 0xBF;   
   // Sets strong pullup active so after the next byte or bit 
   // strong pullup will be active

   if(!I2CBus_write(buffer,2))
   {
      lcd_putchar('f');
   }

   return OWReadBit();  
}

// OWWriteBytePower 
//
// config_byte - current configuration settings.
// wrbyte - byte to be written before the strong pullup is active
// delay - ms delay used before disabling active pullup
//
// Returns failure or success of the operation.
//
uchar OWWriteBytePower(uchar config_byte, uchar wrbyte)
{ 
   uchar buffer[2];
   
   buffer[0] = writeConfigCommand;
   buffer[1] = (config_byte | 0x04) & 0xBF;   
   // Sets strong pullup active so after the next byte or bit 
   // strong pullup will be active

   if(!I2CBus_write(buffer,2))
   {
      return FAILURE;
   }

   OWWriteByte(wrbyte);

   return SUCCESS;
}

// Use to display the byte value
void lcd(uchar byte)
{
   int j, ch;

   for (j = 4; j >= 0; j = j-4)
   {
      ch = (byte >> j) & 0x0F;

      if (ch >= 0x0A)
        lcd_putchar(ch + 'A' - 10);
      else
        lcd_putchar(ch + '0');
   }
}


