//---------------------------------------------------------------------------
// Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//---------------------------------------------------------------------------
//
//  reg420.h - Header file for the registers of the DS89C420.
//
//  Date: Mar 25, 2004
//  Version: 1.00
//
// --------------------------------------------------------------------------

#ifndef __reg420_H
#define __reg420_H

   #ifdef __CPP__
   extern C {
   #endif

      // most definitions copied from Reg5000.h from Keil, copyright below:
      /*--------------------------------------------------------------------------
      REG5000.H

      Header file for Dallas DS5000 microcontroller.
      Copyright (c) 1988-1997 Keil Elektronik GmbH and Keil Software, Inc.
      All rights reserved.
      --------------------------------------------------------------------------*/

      /*  BYTE Register  */
      sfr P0   = 0x80;
      sfr DBB  = 0x80;
      sfr P1   = 0x90;
      sfr P2   = 0xA0;
      sfr P3   = 0xB0;
      sfr PSW  = 0xD0;
      sfr ACC  = 0xE0;
      sfr B    = 0xF0;
      sfr SP   = 0x81;
      sfr DPL  = 0x82;
      sfr DPH  = 0x83;
      sfr PCON = 0x87;
      sfr TCON = 0x88;
      sfr TMOD = 0x89;
      sfr TL0  = 0x8A;
      sfr TL1  = 0x8B;
      sfr TH0  = 0x8C;
      sfr TH1  = 0x8D;
      sfr IE   = 0xA8;
      sfr IP   = 0xB8;
      sfr SCON = 0x98;
      sfr SBUF = 0x99;
      sfr TA	 = 0xC7;
      sfr MCON = 0xC6;
      sfr RPS	 = 0xDA;
      sfr RPCTL= 0xD8;
      sfr RNR  = 0xCF;
      sfr CRC  = 0xC1;


      /*  BIT Register  */
      /*  PSW   */
      sbit CY   = 0xD7;
      sbit AC   = 0xD6;
      sbit F0   = 0xD5;
      sbit RS1  = 0xD4;
      sbit RS0  = 0xD3;
      sbit OV   = 0xD2;
      sbit P    = 0xD0;

      /*  TCON  */
      sbit TF1  = 0x8F;
      sbit TR1  = 0x8E;
      sbit TF0  = 0x8D;
      sbit TR0  = 0x8C;
      sbit IE1  = 0x8B;
      sbit IT1  = 0x8A;
      sbit IE0  = 0x89;
      sbit IT0  = 0x88;

      /*  IE   */
      sbit EA   = 0xAF;
      sbit ES   = 0xAC;
      sbit ET1  = 0xAB;
      sbit EX1  = 0xAA;
      sbit ET0  = 0xA9;
      sbit EX0  = 0xA8;

      /*  IP   */
      sbit PS   = 0xBC;
      sbit PT1  = 0xBB;
      sbit PX1  = 0xBA;
      sbit PT0  = 0xB9;
      sbit PX0  = 0xB8;

      /*  P3  */
      sbit RD   = 0xB7;
      sbit WR   = 0xB6;
      sbit T1   = 0xB5;
      sbit T0   = 0xB4;
      sbit INT1 = 0xB3;
      sbit INT0 = 0xB2;
      sbit TXD  = 0xB1;
      sbit RXD  = 0xB0;

      /*  SCON  */
      sbit SM0  = 0x9F;
      sbit SM1  = 0x9E;
      sbit SM2  = 0x9D;
      sbit REN  = 0x9C;
      sbit TB8  = 0x9B;
      sbit RB8  = 0x9A;
      sbit TI   = 0x99;
      sbit RI   = 0x98;

      // the power managment register for the 420
      sfr PMR = 0xC4;
      sfr EXIF = 0x91;


      /*
      Explanation of different memory locations recognized by C51:
         code - program memory; accessed by opcode MOVC @A+DPTR (Up to 64 KBytes)
         data - directly addressable internal data memory; fastest access to variables (128 bytes)
         idata - indirectly addressable internal data memory; accessed across the full internal address space (256 bytes)
         bdata - bit-addressable internal data memory; allows mixed bit and byte access (16 bytes)
         xdata - external data memory; accessed by opcode MOVX @DPTR (Up to 64 KBytes)
         pdata - paged external data memory; accessed by opcode MOVX @Rn (256 bytes)

      Explanation of different data types and their size for C51:
         char - 1 byte (signed/unsigned)
         enum - 2 bytes
         short - 2 bytes (signed/unsigned)
         int - 2 bytes (signed/unsigned)
         long - 4 bytes (signed/unsigned)
         float - 4 bytes (1.175494E-38 to 3.402823E+38)
         bit - 1 bit
         sbit - 1 bit
         sfr - 1 byte
         sfr16 - 2 bytes
      */

      // COMMONLY USED DATA TYPES, some are specific to Keil C51
      #define uchar unsigned char

      // use to declare a bit-addressable byte
      #define bbyte bdata unsigned char

      // use to declare a bit: DECLARE_BIT(P0, 0, Port0Bit0);
      #define DECLARE_BIT(byteAddress,bitNumber,bitLabel) sbit bitLabel = byteAddress^bitNumber

      // use to declare a scratchpad byte
      #define dbyte data unsigned char
      // use to declare a scratchpad integer
      #define dint data unsigned int
      // use to declare a scratchpad long
      #define dlong data unsigned long

      // use to declare a constant byte
      #define cbyte code unsigned char
      // use to declare a constant integer
      #define cint code unsigned int
      // use to declare a constant long
      #define clong code unsigned long

      // use to declare an XData byte
      #define xbyte xdata unsigned char
      // use to declare an XData int
      #define xint xdata unsigned int
      // use to declare an XData long
      #define xlong xdata unsigned long int

      /*
      ; DS89C420 (/1) @ 11.059MHz = 90 ns cycles
      ; DS89C420 (/1) @ 22.118MHz = 45 ns cycles
      ; DS89C420 (/1) @ 33.000MHz = 30 ns cycles
      */
      #ifndef OSCILLATOR
         #define OSCILLATOR 33000000
      //   #define OSCILLATOR 22118400
      //   #define OSCILLATOR 14745600
      //   #define OSCILLATOR 14318000
      //   #define OSCILLATOR 11059200
      //   #define OSCILLATOR 7370000
      #endif

      // Crystal multiplier for the 420.  By default, the 420 is in 1x mode,
      // but it supports both 2x and 4x multipliers.  BUT, the timers are
      // still looking at the external clock frequency, so we need to know
      // if we are multiplying
      #ifndef CTMDIV
         #ifdef CTM2
            #define CTMDIV 2.0
         #else
            #ifdef CTM4
               #define CTMDIV 4.0
            #else
               #define CTMDIV 1.0
            #endif
         #endif
      #endif

   #ifdef __CPP__
   }
   #endif

#endif