//---------------------------------------------------------------------------
// Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//---------------------------------------------------------------------------
//
//  timer.h - Header file for Timer related routines.
//
//  Date: Mar 25, 2004
//  Version: 1.00
//
// --------------------------------------------------------------------------

#include "reg420.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#ifndef __timer_H
#define __timer_H

   #ifdef __CPP__
   extern C {
   #endif

      // Timer 0 reload value for 1kHz ticks
      // reload value = 65536 - (OSC/12)/1kHz
      // assumes T0DIV=12
      //#define TIMER0_1MSEC_RELOAD ((int)(65536-((OSCILLATOR/CTMDIV/12.0)/1000L)))
      // defined in ecash.h
      //#define TIMER0_1MSEC_RELOAD ((int)(65536-((OSCILLATOR/1.0/12.0)/1000L)))

      extern bbyte Timer_Flags;
      extern bit TimeOut; // Set when Seconds value has unflowed
      extern bit secTimerRun; // Set when TmrSeconds is counting down
      extern bit msTimerRun; // Set when msCounter is counting up
      extern bit msTimerOverflow; // Set when timer runs for more then 255 ms
      extern bit MsgBit; // Message Toggle Flag, set when temp msg is on LCD
      extern bit SecBit; // Toggles every Hz

      extern dbyte TmrSeconds; // Counts down at 1Hz
      extern dbyte TimeOutValue; // used to set the value for timeouts (in seconds)
      extern dbyte msCounter; // Counts up at 1kHz
      extern dbyte QtrSeconds; // counts down each quarter second
      extern dbyte milliseconds; // counts down millseconds

      extern void timer0Init(void);
      extern void startTimerSec(void);
      extern void stopTimerSec(void);
      extern void startTimerMS(void);
      extern void stopTimerMS(void);
      extern void halfUsDelay();
      extern void usDelay(unsigned int);
      extern void msDelay(unsigned int);

   #ifdef __CPP__
   }
   #endif

#endif